@@ -8,6 +8,7 @@ from dataclasses import dataclass, fields
 from decimal import Decimal
 from enum import Enum
 from typing import Generic, Iterable, TypeVar, Union
+from inspect import isclass
 
 import boto3
 from boto3.dynamodb.conditions import Key
@@ -724,8 +725,11 @@ class DynamoCrudRepository(CrudRepository):
             return self.remove_by_id(getattr(to_delete, to_delete.dynamo_id()))
 
     def item_attr2_instance_attr(self, item_attr_val, instance_attr_name: str, instance_attr_type: type):
-        if hasattr(instance_attr_type,  "__supertype__"):
-            instance_attr_type = instance_attr_type.__supertype__
+        if not isclass(instance_attr_type):  # Try to get the class type
+            if hasattr(instance_attr_type,  "__supertype__"):  # Try to get the class type from typings.NewType
+                instance_attr_type = instance_attr_type.__supertype__
+            elif hasattr(instance_attr_type,  "__origin__"):  # Try to get the class type from typings.Generic
+                instance_attr_type = instance_attr_type.__origin__
 
         if ConstantString.ITEM2OBJ_CONV.value in self.map_dict[instance_attr_name]:
             return self.map_dict[instance_attr_name][ConstantString.ITEM2OBJ_CONV.value](item_attr_val)
@@ -733,9 +737,15 @@ class DynamoCrudRepository(CrudRepository):
         elif issubclass(instance_attr_type, (int, float, Decimal)):
             return instance_attr_type(str(item_attr_val))
 
+        elif issubclass(instance_attr_type, bytearray):  # bytearrays are stored as bytes and need to be cast
+            return bytearray(bytes(item_attr_val))
+
         elif issubclass(instance_attr_type, (bytes, bool)):  # Perform a direct conversion
             return instance_attr_type(item_attr_val)
 
+        elif issubclass(instance_attr_type, (set, frozenset, tuple)):  # load the json and cast to set or tuple
+            return instance_attr_type(json.loads(item_attr_val))
+
         elif issubclass(instance_attr_type, (dict, list)):  # json
             return json.loads(item_attr_val)
 
@@ -929,7 +939,7 @@ class DynamoCrudRepository(CrudRepository):
             )  # str cast to support numpy, pandas, etc
 
         elif self.map_dict[instance_attr_name][ConstantString.ITEM_ATTR_TYPE.value] == "B":  # case 'B' (bytes)
-            if isinstance(instance_attr_val, bytes):
+            if isinstance(instance_attr_val, bytes, bytearray):
                 item_attr_val = bytes(instance_attr_val)
             elif isinstance(
                     instance_attr_val, object
@@ -953,6 +963,10 @@ class DynamoCrudRepository(CrudRepository):
             elif isinstance(instance_attr_val, Enum):
                 item_attr_val = instance_attr_val.name
 
+            # sets and tuples (cast to list and converted to json)
+            elif isinstance(instance_attr_val, (set, frozenset, tuple)):
+                item_attr_val = json.dumps(list(instance_attr_val))
+
             # maps and lists (converted to json)
             elif isinstance(instance_attr_val, (dict, list)):
                 item_attr_val = json.dumps(instance_attr_val)
@@ -1099,9 +1113,12 @@ class DynamoCrudRepository(CrudRepository):
 
     @classmethod
     def dynamo_type_from_type(cls, python_type: type):
+        if not isclass(python_type):  # Try to get the class type
+            if hasattr(python_type, "__supertype__"):  # Try to get the class type from typings.NewType
+                python_type = python_type.__supertype__
+            elif hasattr(python_type, "__origin__"):  # Try to get the class type from typings.Generic
+                python_type = python_type.__origin__
         # if using a specific library like numpy or pandas, the user should specify the "N" type himself
-        if hasattr(python_type, "__supertype__"):
-            python_type = python_type.__supertype__
         if issubclass(python_type, (int, float, Decimal)):
             dynamo_type = "N"
         elif issubclass(
@@ -1110,6 +1127,9 @@ class DynamoCrudRepository(CrudRepository):
                         str,
                         dict,
                         list,
+                        set,
+                        frozenset,
+                        tuple,
                         datetime.date,
                         datetime.time,
                         datetime.datetime,
@@ -1118,7 +1138,7 @@ class DynamoCrudRepository(CrudRepository):
         ):
             dynamo_type = "S"
         elif issubclass(
-                python_type, (bytes, object)
+                python_type, (bytes, bytearray, object)
         ):  # general objects will be pickled
             dynamo_type = "B"
         elif issubclass(python_type, bool):
